@@ -15,15 +15,16 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc., 
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * Author: Thomas Wood <thomas.wood@intel.com>
  *
  */
 
 #include "cc-timezone-map.h"
+#include "cc-timezone-location.h"
 #include <math.h>
 #include "tz.h"
 
@@ -59,6 +60,7 @@ struct _CcTimezoneMapPrivate
   gint olsen_map_rowstride;
 
   gdouble selected_offset;
+  gboolean show_offset;
 
   gchar *watermark;
 
@@ -77,6 +79,11 @@ enum
   LAST_SIGNAL
 };
 
+enum {
+  PROP_0,
+  PROP_SELECTED_OFFSET,
+};
+
 static guint signals[LAST_SIGNAL];
 
 
@@ -512,8 +519,12 @@ cc_timezone_map_get_property (GObject    *object,
                               GValue     *value,
                               GParamSpec *pspec)
 {
+  CcTimezoneMap *map = CC_TIMEZONE_MAP(object);
   switch (property_id)
     {
+    case PROP_SELECTED_OFFSET:
+      g_value_set_double(value, map->priv->selected_offset);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -525,8 +536,12 @@ cc_timezone_map_set_property (GObject      *object,
                               const GValue *value,
                               GParamSpec   *pspec)
 {
+  CcTimezoneMap *map = CC_TIMEZONE_MAP(object);
   switch (property_id)
     {
+    case PROP_SELECTED_OFFSET:
+      cc_timezone_map_set_selected_offset(map, g_value_get_double(value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -782,7 +797,7 @@ cc_timezone_map_draw (GtkWidget *widget,
     cairo_stroke(cr);
   }
 
-  if (!priv->location) {
+  if (!priv->show_offset) {
     return TRUE;
   }
 
@@ -813,6 +828,10 @@ cc_timezone_map_draw (GtkWidget *widget,
       g_object_unref (orig_hilight);
     }
 
+  if (!priv->location) {
+    return TRUE;
+  }
+
   /* load pin icon */
   pin = gdk_pixbuf_new_from_file (DATADIR "/pin.png", &err);
 
@@ -822,16 +841,10 @@ cc_timezone_map_draw (GtkWidget *widget,
       g_clear_error (&err);
     }
 
-  GValue lat = {0};
-  GValue lon = {0};
-  g_value_init (&lat, G_TYPE_DOUBLE);
-  g_value_init (&lon, G_TYPE_DOUBLE);
-  g_object_get_property(G_OBJECT (priv->location), "latitude", &lat);
-  g_object_get_property(G_OBJECT (priv->location), "longitude", &lon);
-  pointx = convert_longtitude_to_x (g_value_get_double(&lon), alloc.width);
-  pointy = convert_latitude_to_y (g_value_get_double(&lat), alloc.height);
-  g_value_unset (&lon);
-  g_value_unset (&lat);
+  pointx = convert_longtitude_to_x (
+          cc_timezone_location_get_longitude(priv->location), alloc.width);
+  pointy = convert_latitude_to_y (
+          cc_timezone_location_get_latitude(priv->location), alloc.height);
 
   if (pointy > alloc.height)
     pointy = alloc.height;
@@ -865,6 +878,14 @@ cc_timezone_map_class_init (CcTimezoneMapClass *klass)
   widget_class->realize = cc_timezone_map_realize;
   widget_class->draw = cc_timezone_map_draw;
 
+  g_object_class_install_property(G_OBJECT_CLASS(klass),
+                                  PROP_SELECTED_OFFSET,
+                                  g_param_spec_string ("selected-offset",
+                                      "Selected offset",
+                                      "The selected offset from GMT in hours",
+                                      "",
+                                      G_PARAM_READWRITE));
+
   signals[LOCATION_CHANGED] = g_signal_new ("location-changed",
                                             CC_TYPE_TIMEZONE_MAP,
                                             G_SIGNAL_RUN_FIRST,
@@ -881,17 +902,9 @@ static gint
 sort_locations (CcTimezoneLocation *a,
                 CcTimezoneLocation *b)
 {
-  GValue val_a = {0};
-  GValue val_b = {0};
   gdouble dist_a, dist_b;
-  g_value_init (&val_a, G_TYPE_DOUBLE);
-  g_value_init (&val_b, G_TYPE_DOUBLE);
-  g_object_get_property(G_OBJECT (a), "dist", &val_a);
-  g_object_get_property(G_OBJECT (b), "dist", &val_b);
-  dist_a = g_value_get_double(&val_a);
-  dist_b = g_value_get_double(&val_b);
-  g_value_unset (&val_a);
-  g_value_unset (&val_b);
+  dist_a = cc_timezone_location_get_dist(a);
+  dist_b = cc_timezone_location_get_dist(b);
   if (dist_a > dist_b)
     return 1;
 
@@ -910,14 +923,22 @@ set_location (CcTimezoneMap *map,
 
   priv->location = location;
 
-  info = tz_info_from_location (priv->location);
-
-  priv->selected_offset = tz_location_get_utc_offset (priv->location)
-    / (60.0*60.0) + ((info->daylight) ? -1.0 : 0.0);
+  if (priv->location)
+  {
+    info = tz_info_from_location (priv->location);
+    priv->selected_offset = tz_location_get_utc_offset (priv->location)
+        / (60.0*60.0) + ((info->daylight) ? -1.0 : 0.0);
+    priv->show_offset = TRUE;
+    tz_info_free (info);
+  }
+  else
+  {
+    priv->show_offset = FALSE;
+    priv->selected_offset = 0.0;
+  }
 
   g_signal_emit (map, signals[LOCATION_CHANGED], 0, priv->location);
 
-  tz_info_free (info);
 }
 
 static CcTimezoneLocation *
@@ -934,13 +955,6 @@ get_loc_for_xy (GtkWidget * widget, gint x, gint y)
   GtkAllocation alloc;
   CcTimezoneLocation* location;
 
-  GValue glon = {0};
-  GValue glat = {0};
-  GValue gdist = {0};
-  g_value_init (&glon, G_TYPE_DOUBLE);
-  g_value_init (&glat, G_TYPE_DOUBLE);
-  g_value_init (&gdist, G_TYPE_DOUBLE);
-
   rowstride = priv->visible_map_rowstride;
   pixels = priv->visible_map_pixels;
 
@@ -974,40 +988,34 @@ get_loc_for_xy (GtkWidget * widget, gint x, gint y)
   width = alloc.width;
   height = alloc.height;
 
-  if (x == priv->previous_x && y == priv->previous_y) {
-    priv->distances = g_list_next (priv->distances);
-    location = (CcTimezoneLocation*) priv->distances->data;
-  } else {
-    g_list_free (priv->distances);
-    priv->distances = NULL;
-    for (i = 0; i < array->len; i++)
-      {
-        gdouble pointx, pointy, dx, dy;
-        CcTimezoneLocation *loc = array->pdata[i];
+  if (x == priv->previous_x && y == priv->previous_y) 
+    {
+      priv->distances = g_list_next (priv->distances);
+      location = (CcTimezoneLocation*) priv->distances->data;
+    } else {
+      g_list_free (priv->distances);
+      priv->distances = NULL;
+      for (i = 0; i < array->len; i++)
+        {
+          gdouble pointx, pointy, dx, dy;
+          CcTimezoneLocation *loc = array->pdata[i];
 
-        g_object_get_property(G_OBJECT (loc), "longitude", &glon);
-        g_object_get_property(G_OBJECT (loc), "latitude", &glat);
-        pointx = convert_longtitude_to_x (g_value_get_double(&glon), width);
-        pointy = convert_latitude_to_y (g_value_get_double(&glat), height);
+          pointx = convert_longtitude_to_x (cc_timezone_location_get_longitude(loc), width);
+          pointy = convert_latitude_to_y (cc_timezone_location_get_latitude(loc), height);
 
-        dx = pointx - x;
-        dy = pointy - y;
+          dx = pointx - x;
+          dy = pointy - y;
 
-        g_value_set_double(&gdist, (gdouble) dx * dx + dy * dy);
-        g_object_set_property(G_OBJECT (loc), "dist", &gdist);
-        priv->distances = g_list_prepend (priv->distances, loc);
-      }
-    priv->distances = g_list_sort (priv->distances, (GCompareFunc) sort_locations);
-    location = (CcTimezoneLocation*) priv->distances->data;
-    priv->previous_x = x;
-    priv->previous_y = y;
-  }
+          cc_timezone_location_set_dist(loc, (gdouble) dx * dx + dy * dy);
+          priv->distances = g_list_prepend (priv->distances, loc);
+        }
+      priv->distances = g_list_sort (priv->distances, (GCompareFunc) sort_locations);
+      location = (CcTimezoneLocation*) priv->distances->data;
+      priv->previous_x = x;
+      priv->previous_y = y;
+    }
 
-  g_value_unset (&glon);
-  g_value_unset (&glat);
-  g_value_unset (&gdist);
-
-  return location;
+    return location;
 }
 
 static gboolean
@@ -1119,6 +1127,9 @@ cc_timezone_map_init (CcTimezoneMap *self)
   priv->olsen_map_pixels = gdk_pixbuf_get_pixels (priv->olsen_map);
   priv->olsen_map_rowstride = gdk_pixbuf_get_rowstride (priv->olsen_map);
 
+  priv->selected_offset = 0.0;
+  priv->show_offset = FALSE;
+
   priv->tzdb = tz_load_db ();
 
   g_signal_connect (self, "button-press-event", G_CALLBACK (button_press_event),
@@ -1142,8 +1153,6 @@ cc_timezone_map_set_timezone (CcTimezoneMap *map,
   GPtrArray *locations;
   guint i;
   char *real_tz;
-  GValue zone = {0};
-  g_value_init (&zone, G_TYPE_STRING);
 
   real_tz = g_hash_table_lookup (map->priv->alias_db, timezone);
 
@@ -1152,9 +1161,8 @@ cc_timezone_map_set_timezone (CcTimezoneMap *map,
   for (i = 0; i < locations->len; i++)
     {
       CcTimezoneLocation *loc = locations->pdata[i];
-      g_object_get_property(G_OBJECT (loc), "zone", &zone);
 
-      if (!g_strcmp0 (g_value_get_string(&zone), real_tz ? real_tz : timezone))
+      if (!g_strcmp0 (cc_timezone_location_get_zone(loc), real_tz ? real_tz : timezone))
         {
           set_location (map, loc);
           break;
@@ -1162,7 +1170,6 @@ cc_timezone_map_set_timezone (CcTimezoneMap *map,
     }
 
   gtk_widget_queue_draw (GTK_WIDGET (map));
-  g_value_unset (&zone);
 }
 
 void
@@ -1199,20 +1206,20 @@ cc_timezone_map_get_timezone_at_coords (CcTimezoneMap *map, gdouble lon, gdouble
   if (zone < G_N_ELEMENTS(olsen_map_timezones))
     city = olsen_map_timezones[zone];
 
-  if (city != NULL) {
-    return city;
-  }
-  else {
-    GtkAllocation alloc;
-    GValue val_zone = {0};
-    g_value_init (&val_zone, G_TYPE_STRING);
-    gtk_widget_get_allocation (GTK_WIDGET (map), &alloc);
-    x = convert_longtitude_to_x(lon, alloc.width);
-    y = convert_latitude_to_y(lat, alloc.height);
-    CcTimezoneLocation * loc = get_loc_for_xy(GTK_WIDGET (map), x, y);
-    g_value_unset (&val_zone);
-    return g_value_get_string(&val_zone);
-  }
+  if (city != NULL)
+    {
+      return city;
+    } else {
+      GtkAllocation alloc;
+      GValue val_zone = {0};
+      g_value_init (&val_zone, G_TYPE_STRING);
+      gtk_widget_get_allocation (GTK_WIDGET (map), &alloc);
+      x = convert_longtitude_to_x(lon, alloc.width);
+      y = convert_latitude_to_y(lat, alloc.height);
+      CcTimezoneLocation * loc = get_loc_for_xy(GTK_WIDGET (map), x, y);
+      g_value_unset (&val_zone);
+      return g_value_get_string(&val_zone);
+    }
 }
 
 void
@@ -1225,8 +1232,61 @@ cc_timezone_map_set_watermark (CcTimezoneMap *map, const gchar * watermark)
   gtk_widget_queue_draw (GTK_WIDGET (map));
 }
 
+/**
+ * cc_timezone_map_get_location:
+ * @map: A #CcTimezoneMap
+ *
+ * Returns the current location set for the map.
+ *
+ * Returns: (transfer none): the map location.
+ */
 CcTimezoneLocation *
 cc_timezone_map_get_location (CcTimezoneMap *map)
 {
   return map->priv->location;
 }
+
+/**
+ * cc_timezone_map_clear_location:
+ * @map: A #CcTimezoneMap
+ *
+ * Clear the location currently set for the #CcTimezoneMap. This will remove
+ * the highlight and reset the map to its original state.
+ *
+ */
+void
+cc_timezone_map_clear_location (CcTimezoneMap *map)
+{
+  set_location(map, NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (map));
+}
+
+/**
+ * cc_timezone_map_get_selected_offset:
+ * @map: A #CcTimezoneMap
+ *
+ * Returns the currently selected offset in hours from GMT.
+ *
+ * Returns: The selected offset.
+ *
+ */
+gdouble cc_timezone_map_get_selected_offset(CcTimezoneMap *map)
+{
+  return map->priv->selected_offset;
+}
+
+/**
+ * cc_timezone_map_set_selected_offset:
+ * @map: A #CcTimezoneMap
+ * @offset: The offset from GMT in hours
+ *
+ * Set the currently selected offset for the map and redraw the highlighted
+ * time zone.
+ */
+void cc_timezone_map_set_selected_offset (CcTimezoneMap *map, gdouble offset)
+{
+  map->priv->selected_offset = offset;
+  map->priv->show_offset = TRUE;
+  g_object_notify(G_OBJECT(map), "selected-offset");
+  gtk_widget_queue_draw (GTK_WIDGET (map));
+}
